# Licensed to Elasticsearch B.V. under one or more contributor
# license agreements. See the NOTICE file distributed with
# this work for additional information regarding copyright
# ownership. Elasticsearch B.V. licenses this file to you under
# the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# frozen_string_literal: true

module Atatus
  # @api private
  module Spies
    # @api private
    class ResqueSpy
      TYPE = 'Resque'

      def install
        install_perform_spy
      end

      def install_perform_spy
        if defined?(::Resque) && defined?(::Resque::Job)

          ::Resque::Job.class_eval do
            alias :perform_without_atatus :perform

            def perform
              name = @payload && @payload['class']&.to_s
              transaction = Atatus.start_transaction(name, TYPE)
              perform_without_atatus
              transaction.done 'success'
            rescue ::Exception => e
              Atatus.report(e, handled: false)
              transaction.done 'error' if transaction
              raise
            ensure
              Atatus.end_transaction
            end
          end

        end
      end
    end

    register 'Resque', 'resque', ResqueSpy.new
  end
end
